@@ -1,3 +1,6 @@
+"""
+seldom requests
+"""
 import json
 import warnings
 from typing import Any
@@ -6,6 +9,8 @@ from seldom.running.config import Seldom
 from seldom.logging import log
 from seldom.utils import jsonpath as utils_jsonpath
 from seldom.utils import jmespath as utils_jmespath
+from seldom.utils.curlify import to_curl
+
 
 IMG = ["jpg", "jpeg", "gif", "bmp", "webp"]
 
@@ -21,7 +26,7 @@ def request(func):
 
     def wrapper(*args, **kwargs):
         func_name = func.__name__
-        log.info('\n-------------- Request -----------------[🚀]')
+        log.info('\n-------------- Request -----------------[*]')
         try:
             url = list(args)[1]
         except IndexError:
@@ -34,7 +39,7 @@ def request(func):
         if file_type in IMG:
             img_file = True
 
-        log.info("[method]: {m}      [url]: {u} ".format(m=func_name.upper(), u=url))
+        log.info(f"[method]: {func_name.upper()}      [url]: {url} ")
         auth = kwargs.get("auth", "")
         headers = kwargs.get("headers", "")
         cookies = kwargs.get("cookies", "")
@@ -57,12 +62,13 @@ def request(func):
         # running function
         r = func(*args, **kwargs)
 
+        ResponseResult.request = r.request
         ResponseResult.status_code = r.status_code
-        log.info("-------------- Response ----------------[🛬️]")
+        log.info("\n-------------- Response ----------------[*]")
         if ResponseResult.status_code == 200 or ResponseResult.status_code == 304:
-            log.info("successful with status {}".format(str(ResponseResult.status_code)))
+            log.info(f"successful with status {ResponseResult.status_code}")
         else:
-            log.warning("unsuccessful with status {}".format(str(ResponseResult.status_code)))
+            log.warning(f"unsuccessful with status {ResponseResult.status_code}")
         resp_time = r.elapsed.total_seconds()
         try:
             resp = r.json()
@@ -70,12 +76,13 @@ def request(func):
             log.debug(f"[response]:\n {formatting(resp)}")
             ResponseResult.response = resp
         except BaseException as msg:
-            log.debug(f"[warning]: failed to convert res to json, try to convert to text")
+            log.debug("[warning]: failed to convert res to json, try to convert to text")
             log.trace(f"[warning]: {msg}")
             if img_file is True:
                 log.debug(f"[type]: {file_type}      [time]: {resp_time}")
                 ResponseResult.response = r.content
             else:
+                r.encoding = 'utf-8'
                 log.debug(f"[type]: text      [time]: {resp_time}")
                 log.debug(f"[response]:\n {r.text}")
                 ResponseResult.response = r.text
@@ -88,9 +95,11 @@ def request(func):
 class ResponseResult:
     status_code = 200
     response = None
+    request = None
 
 
-class HttpRequest(object):
+class HttpRequest:
+    """seldom http request class"""
 
     @request
     def get(self, url, params=None, **kwargs):
@@ -178,6 +187,20 @@ class HttpRequest(object):
         log.debug(f"[jresponse]:\n {str(ret)}")
         return ret
 
+    @staticmethod
+    def curl(request=None, compressed: bool = False, verify: bool = True) -> str:
+        """
+        requests to cURL command
+        :param request: request object
+        :param compressed:
+        :param verify:
+        :return:
+        """
+        if request is None:
+            return to_curl(ResponseResult.request, compressed, verify)
+
+        return to_curl(request, compressed, verify)
+
     class Session(requests.Session):
 
         @request
@@ -282,8 +305,8 @@ def check_response(describe: str = "", status_code: int = 200, ret: str = None,
                 if data is None:
                     log.error(f"Execute {func_name} - return {ret} is None")
                 return data
-            else:
-                return r.json()
+
+            return r.json()
 
         return wrapper
 
