@@ -1,3 +1,6 @@
+"""
+unittest decorator
+"""
 import unittest
 import functools
 
@@ -53,16 +56,20 @@ def depend(case=None):
     :return:
     """
     def wrapper_func(test_func):
+
         @functools.wraps(test_func)
-        def inner_func(self):
+        def inner_func(self, *args):
             if case == test_func.__name__:
-                raise ValueError("{} cannot depend on itself".format(case))
+                raise ValueError(f"{case} cannot depend on itself")
             failures = str([fail_[0] for fail_ in self._outcome.result.failures])
             errors = str([error_[0] for error_ in self._outcome.result.errors])
             skipped = str([skip_[0] for skip_ in self._outcome.result.skipped])
             flag = (case in failures) or (case in errors) or (case in skipped)
-            test = skip_if(flag, '{} failed  or  error or skipped'.format(case))(test_func)
-            return test(self)
+            test = skip_if(flag, f'{case} failed  or  error or skipped')(test_func)
+            try:
+                return test(self)
+            except TypeError:
+                return None
         return inner_func
     return wrapper_func
 
